<?php

namespace General\Requests\CompanyAuth;

use General\Requests\Request as RequestsRequest;
use Illuminate\Http\Exceptions\HttpResponseException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class CompanyAuthRequest extends RequestsRequest
{
    public function __construct(Request $request)
    {
        $this->input = $request->all();
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            'app_id' => 'required|string|exists:company_auth_configs',
            'nonce_str' => 'required|string',
            'timestamp' => 'required|integer',
            'sign' => 'required|string',
        ];
    }

    public function messages()
    {
        return array_merge(parent::messages(),  [
            'app_id.exists' => '未授权!',
        ]);
    }

    public function validate()
    {
        $validator = Validator::make($this->input, $this->rules(), $this->messages());
        if ($validator->fails()) {
            $err = $validator->errors()->all();
            throw (new HttpResponseException(response()->json([
                'code' => -1,
                'msg' => $validator->errors()->first(),
                'data' => $validator->errors()->first(),
            ], 200)));;
        }

        return true;
    }
}
