<?php

/**
 * Created by PhpStorm.
 * User: tandunzhao
 * Date: 2017/12/4
 * Time: 下午1:43
 */

namespace General\Services\OfficialAccount;

use General\Models\OfficialAccount\CustomSendMsgs;
use General\Models\OfficialAccount\ForceSubscribeDelayMsg;
use General\Models\OfficialAccount\OfficialAccount;
use General\Models\OfficialAccount\WechatTemplateMsgs;
use General\Models\Order\Order;
use General\Models\Statistic\WapVisitStat;
use Illuminate\Support\Facades\DB;
use Redis;

class OfficialAccountService
{
    public function getOfficialAccountsByChannelIds(array $channel_ids)
    {
        return OfficialAccount::whereIn('distribution_channel_id', $channel_ids)->get();
    }

    /**
     * 查询客服消息
     */
    public function findCustomSendMsgs(array $params, bool $is_all = false)
    {
        $res = CustomSendMsgs::where('distribution_channel_id', $params['channel_id'])
            ->where('del_flag', 0)
            ->where('is_show_list', 1);
        //创建时间
        if (isset($params['start_time']) && $params['start_time']) {
            $res->where('created_at', '>=', $params['start_time']);
        }
        if (isset($params['end_time']) && $params['end_time']) {
            $res->where('created_at', '<=', $params['end_time']);
        }
        //发送时间
        if (isset($params['send_start_time']) && $params['send_start_time']) {
            $res->where('send_time', '>=', $params['send_start_time']);
        }
        if (isset($params['send_end_time']) && $params['send_end_time']) {
            $res->where('send_time', '<=', $params['send_end_time']);
        }
        if (isset($params['book_name']) && $params['book_name']) {
            $res->where('book_name', 'like', $params['book_name'] . '%');
        }
        if (isset($params['name']) && $params['name']) {
            $res->where('name', 'like', $params['task_name'] . '%');
        }
        if ($is_all) {
            $custom_send_msgs = $res->orderBy('id', 'desc')->get();
        } else {
            $custom_send_msgs = $res->orderBy('id', 'desc')->paginate();
        }
        return $this->setStatisticInfo($custom_send_msgs, 'custom_');
    }

    public function setStatisticInfo($data, string $left_str)
    {
        foreach ($data as $key => $item) {
            $fromtype = $left_str . $item->id;
            $statistics = $this->innerCustomerAllStats($fromtype);
            $data[$key]->uv = isset($statistics['uv']) ? $statistics['uv'] : '';
            $data[$key]->pv = isset($statistics['pv']) ? $statistics['pv'] : '';
            $data[$key]->pay_user_num = isset($statistics['pay_user_num']) ? $statistics['pay_user_num'] : '';
            $data[$key]->charge_amount = isset($statistics['charge_amount']) ? $statistics['charge_amount'] : '';
        }
        return $data;
    }

    /**
     * 内部派单统计
     * @param $from
     * @return array
     */
    public function innerCustomerAllStats($from)
    {
        $uv = Redis::scard('push:inner_send_order_id:uv:' . $from);
        $info = DB::table('wap_long_visit_stats')->where('stats_type', 'inner_send_order_id')->where('param', $from)
            ->select(DB::raw('sum(pv) as pv,sum(uv) as uv'))->first();
        $uv_db = 0;
        $pv_db = 0;
        if ($info) {
            $pv_db = $info->pv;
            $uv_db = $info->uv;
        }
        $pv = Redis::hget('push:inner_send_order_id:pv', $from) + $pv_db;
        $order_info = $this->getInnerSendOrderStats($from);
        $pay_user_num = 0;
        $charge_amount = 0;
        if ($uv_db) {
            $uv = (int) $uv + $uv_db;
        }
        if ($order_info) {
            $pay_user_num = $order_info->pay_user_num;
            $charge_amount = $order_info->charge_amount;
        }
        return compact('uv', 'pv', 'pay_user_num', 'charge_amount');
    }

    public function getInnerSendOrderStats($inner_send_order_id)
    {
        return Order::where('inner_send_order_id', $inner_send_order_id)
            ->select(
                DB::raw('count(distinct uid) as pay_user_num'),
                DB::raw('sum(price) as charge_amount')
            )
            ->where('status', 'PAID')
            ->first();
    }

    /**
     * 查询模板消息
     */
    public function findWechatTemplateMsgs($param, $is_all = false)
    {
        $res = WechatTemplateMsgs::where('distribution_channel_id', $param['channel_id'])
            ->where('is_show_list', 1);
        if (isset($param['is_delete']) && 1 == $param['is_delete']) {
            $res->where('del_flag', 1);
        } else {
            $res->where('del_flag', 0);
        }

        if (isset($param['start_time']) && $param['start_time']) {
            $res->where('created_at', '>=', $param['start_time']);
        }
        if (isset($param['end_time']) && $param['end_time']) {
            $res->where('created_at', '<=', $param['end_time']);
        }
        if (isset($param['book_name']) && $param['book_name']) {
            $res->where('book_name', 'like', '%' . $param['book_name'] . '%');
        }
        if (isset($param['task_name']) && $param['task_name']) {
            $res->where('name', 'like', '%' . $param['task_name'] . '%');
        }

        if ($is_all) {
            $wechat_template_msgs = $res->orderBy('id', 'desc')->get();
        } else {
            $wechat_template_msgs = $res->orderBy('id', 'desc')->paginate();
        }
        return $this->setStatisticInfo($wechat_template_msgs, 'template_');
    }

    /**
     * 延时消息
     */
    public function findWechatDelayMsgs(array $param, $is_all = false)
    {
        $res = ForceSubscribeDelayMsg::where('distribution_channel_id', $param['channel_id'])
            ->where('is_show', 1);
        if ($is_all) {
            $data = $res->orderBy('id', 'desc')->get();
        } else {
            $data = $res->orderBy('id', 'desc')->paginate();
        }
        foreach ($data as $key => $item) {
            $fromtype = 'subscribe_delay_' . $item->id;
            $statistics = $this->getSitePvAndUv($param['channel_id'], $fromtype);
            $data[$key]->uv = isset($statistics['uv']) ? $statistics['uv'] : '';
            $data[$key]->pv = isset($statistics['pv']) ? $statistics['pv'] : '';
            $data[$key]->pay_user_num = isset($statistics['pay_user_num']) ? $statistics['pay_user_num'] : '';
            $data[$key]->charge_amount = isset($statistics['charge_amount']) ? $statistics['charge_amount'] : '';
        }
        return $data;
    }

    /**
     *获取site下的uv,pv
     */
    public function getSitePvAndUv(int $distribution_channel_id, string $from)
    {
        $res = WapVisitStat::where('key', $distribution_channel_id)
            ->where('type', 1)
            ->where('from_type', $from)
            ->select(DB::raw('sum(uv) as uv'), DB::raw('sum(pv) as pv'))
            ->get();
        $temp = $res->first();
        $pv = $temp->pv;
        $uv = $temp->uv;
        $charge_amount = (float)Redis::hget('push:all:paidamount', $from);
        $pay_user_num = (int)Redis::scard('push:all:paidnum:from:' . $from);
        $now = date('Y-m-d');
        $pv += (int)Redis::hget('customer:push:click:distribution_channel_id:' . $distribution_channel_id . 'from:' . $from, $now);
        $uv += (int)Redis::scard('push:distribution_channel_id:' . $distribution_channel_id . 'from:' . $from . ':date:' . $now);
        return compact('uv', 'pv', 'pay_user_num', 'charge_amount');
    }
}
